from openai.types.chat import ChatCompletionContentPartTextParam, ChatCompletionContentPartImageParam, \
    ChatCompletionContentPartInputAudioParam
from openai.types.chat.chat_completion_content_part_image_param import ImageURL
from openai.types.chat.chat_completion_content_part_input_audio_param import InputAudio

from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.message import Message, ContentType, Content
from llm_bridge.type.model_message.gpt_message import GptMessage


async def convert_message_to_gpt(message: Message) -> GptMessage:
    role = message.role
    content = []

    for content_item in message.contents:
        if content_item.type == ContentType.Text:
            text_content = ChatCompletionContentPartTextParam(type="text", text=content_item.data)
            content.append(text_content)
        elif content_item.type == ContentType.File:
            file_url = content_item.data
            file_type, sub_type = await get_file_type(file_url)
            if file_type == "image":
                image_url = await media_processor.get_gpt_image_content_from_url(file_url)
                image_content = ChatCompletionContentPartImageParam(
                    type="image_url",
                    image_url=ImageURL(url=image_url)
                )
                content.append(image_content)
            elif file_type == "audio":
                encoded_string = await media_processor.get_gpt_audio_content_from_url(file_url)
                audio_content = ChatCompletionContentPartInputAudioParam(
                    type="input_audio",
                    input_audio=InputAudio(data=encoded_string, format=sub_type)
                )
                content.append(audio_content)
            else:
                text_content = ChatCompletionContentPartTextParam(
                    type="text",
                    text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
                )
                content.append(text_content)

    return GptMessage(role=role, content=content)



from dataclasses import dataclass

from openai.types.chat import (
    ChatCompletionContentPartTextParam,
    ChatCompletionContentPartImageParam, \
    ChatCompletionContentPartInputAudioParam
)

from llm_bridge.type.message import Role


@dataclass
class GptMessage:
    role: Role
    content: list[
        ChatCompletionContentPartTextParam |
        ChatCompletionContentPartImageParam |
        ChatCompletionContentPartInputAudioParam
    ]



import base64

from llm_bridge.logic.file_fetch import fetch_file_data
from llm_bridge.type.model_message import claude_message


async def get_gpt_image_content_from_url(req_img_url: str) -> str:
    img_data, media_type = await fetch_file_data(req_img_url)
    base64_image = base64.b64encode(img_data.getvalue()).decode('utf-8')
    return f"data:{media_type};base64,{base64_image}"


async def get_gemini_image_content_from_url(req_img_url: str) -> tuple[bytes, str]:
    img_data, media_type = await fetch_file_data(req_img_url)
    img_bytes = img_data.getvalue()
    return img_bytes, media_type


async def get_claude_image_content_from_url(req_img_url: str) -> claude_message.ImageContent:
    img_data, media_type = await fetch_file_data(req_img_url)
    base64_image = base64.b64encode(img_data.getvalue()).decode('utf-8')
    image_source = claude_message.ImageSource(type="base64", media_type=media_type, data=base64_image)
    return claude_message.ImageContent(type="image", source=image_source)


async def get_gpt_audio_content_from_url(req_audio_url: str) -> str:
    audio_data, media_type = await fetch_file_data(req_audio_url)
    audio_bytes = audio_data.getvalue()
    encoded_string = base64.b64encode(audio_bytes).decode('utf-8')
    return encoded_string


async def get_gemini_audio_content_from_url(req_audio_url: str) -> tuple[bytes, str]:
    audio_data, media_type = await fetch_file_data(req_audio_url)
    audio_bytes = audio_data.getvalue()
    if media_type == 'video/webm':
        media_type = 'audio/webm'
    return audio_bytes, media_type



import logging
from io import BytesIO

import docx
import fitz
import httpx
import openpyxl
from docx.oxml import CT_P, CT_Tbl
from fastapi import HTTPException
from pptx import Presentation

from llm_bridge.logic.file_fetch import fetch_file_data
from llm_bridge.logic.message_preprocess import file_type_checker


async def extract_text_from_file(file_url: str) -> str:
    content_bytes, content_type = await fetch_file_data(file_url)
    file_content = content_bytes.getvalue()

    if file_type_checker.is_file_type_supported(file_url) is False:
        raise HTTPException(status_code=415, detail="legacy filetypes ('.doc', '.xls', '.ppt') are not supported")
    file_type, sub_type = await file_type_checker.get_file_type(file_url)
    if sub_type == "code":
        return extract_text_from_code(file_content)
    if sub_type == "pdf":
        return extract_text_from_pdf(file_content)
    if sub_type == "word":
        return extract_text_from_word(file_content)
    if sub_type == "excel":
        return extract_text_from_excel(file_content)
    if sub_type == "ppt":
        return extract_text_from_ppt(file_content)

    try:
        return file_content.decode('utf-8')
    except UnicodeDecodeError as e:
        logging.exception(e)
        raise HTTPException(status_code=415, detail=str(e))


def extract_text_from_code(file_content: bytes) -> str:
    return file_content.decode('utf-8')


def extract_text_from_pdf(file_content: bytes) -> str:
    text = ""
    with fitz.open(stream=file_content, filetype="pdf") as doc:
        for page in doc:
            text += page.get_text()
    return text


def extract_text_from_word(file_content: bytes) -> str:
    text = ""
    try:
        doc = docx.Document(BytesIO(file_content))

        # Extract text from headers
        for section in doc.sections:
            header = section.header
            for para in header.paragraphs:
                text += para.text + "\n"

        # Extract text from the main document body
        for element in doc.element.body:
            if isinstance(element, CT_P):
                para = docx.text.paragraph.Paragraph(element, doc)
                text += para.text + "\n"
            elif isinstance(element, CT_Tbl):
                table = docx.table.Table(element, doc)
                for row in table.rows:
                    for cell in row.cells:
                        text += cell.text + "\t"
                    text += "\n"

        # Extract text from footers
        for section in doc.sections:
            footer = section.footer
            for para in footer.paragraphs:
                text += para.text + "\n"

        return text
    except Exception as e:
        logging.exception(e)
        raise HTTPException(status_code=415, detail=str(e))


def extract_text_from_excel(file_content: bytes) -> str:
    text = ""
    try:
        workbook = openpyxl.load_workbook(BytesIO(file_content), data_only=True)
        for sheet in workbook:
            text += f"Sheet: {sheet.title}\n"
            for row in sheet.iter_rows(values_only=True):
                text += "\t".join([str(cell) if cell is not None else "" for cell in row]) + "\n"
        return text
    except Exception as e:
        logging.exception(e)
        raise HTTPException(status_code=415, detail=str(e))


def extract_text_from_ppt(file_content: bytes) -> str:
    text = ""
    try:
        presentation = Presentation(BytesIO(file_content))
        for slide in presentation.slides:
            for shape in slide.shapes:
                if hasattr(shape, "text"):
                    text += shape.text + "\n"
        return text
    except Exception as e:
        logging.exception(e)
        raise HTTPException(status_code=415, detail=str(e))



import pytest

from llm_bridge.logic.chat_generate.chat_client_factory import create_chat_client
from llm_bridge.type.message import Message, Role, Content, ContentType


@pytest.fixture
def sample_messages():
    return [
        Message(role=Role.System, contents=[
            Content(type=ContentType.Text, data="You are a helpful assistant.")
        ]),
        Message(role=Role.User, contents=[
            Content(type=ContentType.Text, data="Hello")
        ])
    ]


@pytest.mark.asyncio
async def test_create_gpt_client_openai():
    messages = [Message(role=Role.User, contents=[
        Content(type=ContentType.Text, data="Hello")
    ])]
    model = "gpt-4o"
    api_type = "OpenAI"
    temperature = 0
    stream = False
    api_keys = {
        "OPENAI_API_KEY": "test-key"
    }

    client = await create_chat_client(
        messages=messages,
        model=model,
        api_type=api_type,
        temperature=temperature,
        stream=stream,
        api_keys=api_keys
    )

    assert True
